
#pragma once

#include <stdio.h>
#include <stdlib.h>

#include <list>
#include <vector>
#include <map>
#include <set>
#include <stack>
#include <queue>
#include <string>

#include <assert.h>

// don't enable memory checking for code we're building in a static library
// otherwise we'd get 2 different delete functions in it and it wouldn't link
#ifdef _LIB
#	define CLEAN_MEM_DISABLE
#endif

#ifdef CLEAN_MEM_DISABLE

	extern void BeginCheckingUnfreed(void);
	extern void DumpUnfreed(void);

#else // !CLEAN_MEM_DISABLE:

///  #include <condefs.h>
//  #include <fstream.h>
//  #include <iostream.h>
//  #include <iomanip.h>

#pragma warning(disable: 4311)
#pragma warning(disable: 4267)
#pragma warning(disable: 4291)



	// make sure that there's a filename specified for the log file...
#	ifndef CLEAN_MEM_LOG_FILE
#		define CLEAN_MEM_LOG_FILE "leaks.log"
#	endif

	struct ALLOC_INFO;
	typedef std::vector<ALLOC_INFO*> AllocList;
	extern AllocList *allocList;
	extern char * allocDeleteFile;
	extern char * allocDeleteFunction;
	extern int allocDeleteLine;
	extern void OutputDebugString(char *debugStr);
	extern void BeginCheckingUnfreed(void);
	extern void DumpUnfreed(void);
	extern void AddTrack(unsigned long addr,  unsigned long asize,  const char *fname, const char *funcName, unsigned long lnum, bool array);
	extern void RemoveTrack(unsigned long addr, bool array);
	extern void * operator new(size_t size, const char *file, int line, const char *function);
	extern void * operator new[](size_t size, const char *file, int line, const char *function);
	extern void operator delete(void * mem);
	extern void operator delete[](void * mem);
	extern void * __malloc(size_t size, const char *file, int line, const char *function);
	extern void __free(void * mem);
	extern bool checkLeakScope;
	extern bool leakDebugFileOpened;

	// macro stuff for putting in file & line #

#	define new new(__FILE__, __LINE__, __FUNCTION__)

#	ifndef CLEAN_MEM_NO_VERIFY

		// for some reason delete cannot be overrided as easily as new can
		// so here's a tricky macro to pass the information anyway
#		define delete allocDeleteLine = __LINE__, allocDeleteFile = __FILE__, allocDeleteFunction = __FUNCTION__, delete

#	endif // !CLEAN_MEM_NO_VERIFY

#	define malloc(s) __malloc(s, __FILE__, __LINE__, __FUNCTION__)
#	define free(p) __free(p)


#endif // !CLEAN_MEM_DISABLE


